import { readFile } from 'node:fs/promises';
import { fileSyntax, getDefaultSassImplementation, resolveCanonicalize } from './sass-utils.js';
let sassEngine;
import * as fs from 'node:fs';
import path from 'node:path';
import { fileURLToPath, pathToFileURL } from 'node:url';
export const transformSass = async (filePath, options)=>{
    const { sourcemap } = options;
    if (!sassEngine) {
        try {
            sassEngine = getDefaultSassImplementation();
        } catch (e) {
            console.error(e);
            process.exit(1);
        }
    }
    const syntax = fileSyntax(filePath);
    const basedir = path.dirname(filePath);
    const contents = await readFile(filePath, 'utf-8');
    const warnings = [];
    const alias = options.alias || {};
    const result = await sassEngine.compileStringAsync(contents, {
        sourceMapIncludeSources: true,
        sourceMap: sourcemap,
        syntax,
        alertColor: false,
        logger: {
            warn (message, options) {
                if (!options.span) {
                    warnings.push({
                        text: `sass warning: ${message}`
                    });
                } else {
                    var _options_span_url;
                    var _options_span_url_pathname;
                    const filename = (_options_span_url_pathname = (_options_span_url = options.span.url) === null || _options_span_url === void 0 ? void 0 : _options_span_url.pathname) !== null && _options_span_url_pathname !== void 0 ? _options_span_url_pathname : path;
                    const esbuildMsg = {
                        text: message,
                        location: {
                            file: filename,
                            line: options.span.start.line,
                            column: options.span.start.column,
                            lineText: options.span.text
                        },
                        detail: {
                            deprecation: options.deprecation,
                            stack: options.stack
                        }
                    };
                    warnings.push(esbuildMsg);
                }
            }
        },
        importer: {
            load (canonicalUrl) {
                const pathname = fileURLToPath(canonicalUrl);
                const contents = fs.readFileSync(pathname, 'utf8');
                return {
                    contents,
                    syntax: fileSyntax(pathname),
                    sourceMapUrl: sourcemap ? canonicalUrl : undefined
                };
            },
            canonicalize (url) {
                const urlPath = resolveCanonicalize(filePath, url, alias);
                if (urlPath) {
                    return pathToFileURL(urlPath);
                }
                return null;
            }
        }
    });
    const sourceMap = result.sourceMap;
    if (sourceMap) {
        sourceMap.sourceRoot = basedir;
        sourceMap.sources = sourceMap.sources.map((source)=>{
            return path.relative(basedir, source.startsWith('data:') ? filePath : fileURLToPath(source));
        });
    }
    const imports = result.loadedUrls.map((url)=>fileURLToPath(url));
    return {
        css: result.css,
        map: result.sourceMap ? JSON.stringify(result.sourceMap) : '',
        imports,
        warnings
    };
};
